@@ -120,6 +120,47 @@
       return ans;
     };
 
+    BookIndexing.prototype.all_links_and_anchors = function() {
+      var a, anchors, body, br, geom, href, id, left, links, name, top, _i, _len, _ref, _ref1, _ref2, _ref3;
+      body = document.body;
+      links = [];
+      anchors = {};
+      _ref = document.querySelectorAll("body a[href], body [id], body a[name]");
+      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
+        a = _ref[_i];
+        if ((_ref1 = window.paged_display) != null ? _ref1.in_paged_mode : void 0) {
+          geom = window.paged_display.column_location(a);
+        } else {
+          br = a.getBoundingClientRect();
+          _ref2 = viewport_to_document(br.left, br.top, a.ownerDocument), left = _ref2[0], top = _ref2[1];
+          geom = {
+            'left': left,
+            'top': top,
+            'width': br.right - br.left,
+            'height': br.bottom - br.top
+          };
+        }
+        href = a.getAttribute('href');
+        if (href) {
+          links.push([href, geom]);
+        }
+        id = a.getAttribute("id");
+        if (id && __indexOf.call(anchors, id) < 0) {
+          anchors[id] = geom;
+        }
+        if ((_ref3 = a.tagName) === 'A' || _ref3 === "a") {
+          name = a.getAttribute("name");
+          if (name && __indexOf.call(anchors, name) < 0) {
+            anchors[name] = geom;
+          }
+        }
+      }
+      return {
+        'links': links,
+        'anchors': anchors
+      };
+    };
+
     return BookIndexing;
 
   })();
