@@ -624,12 +624,6 @@ from calibre.library.catalogs.epub_mobi import EPUB_MOBI
 plugins += [CSV_XML, BIBTEX, EPUB_MOBI]
 # }}}
 
-# EPUB Fix plugins {{{
-from calibre.ebooks.epub.fix.unmanifested import Unmanifested
-from calibre.ebooks.epub.fix.epubcheck import Epubcheck
-plugins += [Unmanifested, Epubcheck]
-# }}}
-
 # Profiles {{{
 from calibre.customize.profiles import input_profiles, output_profiles
 plugins += input_profiles + output_profiles
@@ -640,7 +634,7 @@ from calibre.devices.apple.driver import ITUNES
 from calibre.devices.hanlin.driver import HANLINV3, HANLINV5, BOOX, SPECTRA
 from calibre.devices.blackberry.driver import BLACKBERRY, PLAYBOOK
 from calibre.devices.cybook.driver import CYBOOK, ORIZON
-from calibre.devices.eb600.driver import (EB600, COOL_ER, SHINEBOOK,
+from calibre.devices.eb600.driver import (EB600, COOL_ER, SHINEBOOK, TOLINO,
                 POCKETBOOK360, GER2, ITALICA, ECLICTO, DBOOK, INVESBOOK,
                 BOOQ, ELONEX, POCKETBOOK301, MENTOR, POCKETBOOK602,
                 POCKETBOOK701, POCKETBOOK360P, PI2, POCKETBOOK622)
@@ -710,7 +704,7 @@ plugins += [
     INVESBOOK,
     BOOX,
     BOOQ,
-    EB600,
+    EB600, TOLINO,
     README,
     N516, KIBANO,
     THEBOOK, LIBREAIR,
@@ -763,8 +757,10 @@ from calibre.ebooks.metadata.sources.isbndb import ISBNDB
 from calibre.ebooks.metadata.sources.overdrive import OverDrive
 from calibre.ebooks.metadata.sources.douban import Douban
 from calibre.ebooks.metadata.sources.ozon import Ozon
+from calibre.ebooks.metadata.sources.google_images import GoogleImages
+from calibre.ebooks.metadata.sources.big_book_search import BigBookSearch
 
-plugins += [GoogleBooks, Amazon, Edelweiss, OpenLibrary, ISBNDB, OverDrive, Douban, Ozon]
+plugins += [GoogleBooks, GoogleImages, Amazon, Edelweiss, OpenLibrary, ISBNDB, OverDrive, Douban, Ozon, BigBookSearch]
 
 # }}}
 
@@ -790,11 +786,16 @@ class ActionConvert(InterfaceActionBase):
     actual_plugin = 'calibre.gui2.actions.convert:ConvertAction'
     description = _('Convert books to various ebook formats')
 
-# class ActionPolish(InterfaceActionBase):
-#     name = 'Polish Books'
-#     actual_plugin = 'calibre.gui2.actions.polish:PolishAction'
-#     description = _('Fine tune your ebooks')
-#
+class ActionPolish(InterfaceActionBase):
+    name = 'Polish Books'
+    actual_plugin = 'calibre.gui2.actions.polish:PolishAction'
+    description = _('Fine tune your ebooks')
+
+class ActionEditToC(InterfaceActionBase):
+    name = 'Edit ToC'
+    actual_plugin = 'calibre.gui2.actions.toc_edit:ToCEditAction'
+    description = _('Edit the Table of Contents in your books')
+
 class ActionDelete(InterfaceActionBase):
     name = 'Remove Books'
     actual_plugin = 'calibre.gui2.actions.delete:DeleteAction'
@@ -930,12 +931,12 @@ class ActionPluginUpdater(InterfaceActionBase):
 
 plugins += [ActionAdd, ActionFetchAnnotations, ActionGenerateCatalog,
         ActionConvert, ActionDelete, ActionEditMetadata, ActionView,
-        ActionFetchNews, ActionSaveToDisk, ActionQuickview, #ActionPolish,
+        ActionFetchNews, ActionSaveToDisk, ActionQuickview, ActionPolish,
         ActionShowBookDetails,ActionRestart, ActionOpenFolder, ActionConnectShare,
         ActionSendToDevice, ActionHelp, ActionPreferences, ActionSimilarBooks,
         ActionAddToLibrary, ActionEditCollections, ActionChooseLibrary,
         ActionCopyToLibrary, ActionTweakEpub, ActionNextMatch, ActionStore,
-        ActionPluginUpdater, ActionPickRandom]
+        ActionPluginUpdater, ActionPickRandom, ActionEditToC]
 
 # }}}
 
@@ -1297,15 +1298,6 @@ class StoreBeamEBooksDEStore(StoreBase):
     formats = ['EPUB', 'MOBI', 'PDF']
     affiliate = True
 
-class StoreBeWriteStore(StoreBase):
-    name = 'BeWrite Books'
-    description = u'Publishers of fine books. Highly selective and editorially driven. Does not offer: books for children or exclusively YA, erotica, swords-and-sorcery fantasy and space-opera-style science fiction. All other genres are represented.'
-    actual_plugin = 'calibre.gui2.store.stores.bewrite_plugin:BeWriteStore'
-
-    drm_free_only = True
-    headquarters = 'US'
-    formats = ['EPUB', 'MOBI', 'PDF']
-
 class StoreBiblioStore(StoreBase):
     name = u'Библио.бг'
     author = 'Alex Stanev'
@@ -1476,6 +1468,17 @@ class StoreKoboStore(StoreBase):
     formats = ['EPUB']
     affiliate = True
 
+class StoreKoobeStore(StoreBase):
+    name = 'Koobe'
+    author = u'Tomasz Długosz'
+    description = u'Księgarnia internetowa oferuje ebooki (książki elektroniczne) w postaci plików epub, mobi i pdf.'
+    actual_plugin = 'calibre.gui2.store.stores.koobe_plugin:KoobeStore'
+
+    drm_free_only = True
+    headquarters = 'PL'
+    formats = ['EPUB', 'MOBI', 'PDF']
+    affiliate = True
+
 class StoreLegimiStore(StoreBase):
     name = 'Legimi'
     author = u'Tomasz Długosz'
@@ -1658,6 +1661,7 @@ class StoreWoblinkStore(StoreBase):
 
     headquarters = 'PL'
     formats = ['EPUB', 'MOBI', 'PDF', 'WOBLINK']
+    affiliate = True
 
 class XinXiiStore(StoreBase):
     name = 'XinXii'
@@ -1667,15 +1671,6 @@ class XinXiiStore(StoreBase):
     headquarters = 'DE'
     formats = ['EPUB', 'PDF']
 
-class StoreZixoStore(StoreBase):
-    name = 'Zixo'
-    author = u'Tomasz Długosz'
-    description = u'Księgarnia z ebookami oraz książkami audio. Aby otwierać książki w formacie Zixo należy zainstalować program dostępny na stronie księgarni. Umożliwia on m.in. dodawanie zakładek i dostosowywanie rozmiaru czcionki.'
-    actual_plugin = 'calibre.gui2.store.stores.zixo_plugin:ZixoStore'
-
-    headquarters = 'PL'
-    formats = ['PDF, ZIXO']
-
 plugins += [
     StoreArchiveOrgStore,
     StoreAmazonKindleStore,
@@ -1687,7 +1682,6 @@ plugins += [
     StoreBaenWebScriptionStore,
     StoreBNStore,
     StoreBeamEBooksDEStore,
-    StoreBeWriteStore,
     StoreBiblioStore,
     StoreBookotekaStore,
     StoreChitankaStore,
@@ -1705,6 +1699,7 @@ plugins += [
     StoreGoogleBooksStore,
     StoreGutenbergStore,
     StoreKoboStore,
+    StoreKoobeStore,
     StoreLegimiStore,
     StoreLibreDEStore,
     StoreLitResStore,
@@ -1725,8 +1720,7 @@ plugins += [
     StoreWeightlessBooksStore,
     StoreWHSmithUKStore,
     StoreWoblinkStore,
-    XinXiiStore,
-    StoreZixoStore
+    XinXiiStore
 ]
 
 # }}}
