@@ -7,21 +7,14 @@ __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import shutil, unittest, tempfile, datetime
-from cStringIO import StringIO
+import datetime
+from io import BytesIO
 
 from calibre.utils.date import utc_tz
 from calibre.db.tests.base import BaseTest
 
 class ReadingTest(BaseTest):
 
-    def setUp(self):
-        self.library_path = tempfile.mkdtemp()
-        self.create_db(self.library_path)
-
-    def tearDown(self):
-        shutil.rmtree(self.library_path)
-
     def test_read(self): # {{{
         'Test the reading of data from the database'
         cache = self.init_cache(self.library_path)
@@ -55,7 +48,7 @@ class ReadingTest(BaseTest):
                     '#tags':(),
                     '#yesno':None,
                     '#comments': None,
-
+                    'size':None,
                 },
 
                 2 : {
@@ -66,7 +59,7 @@ class ReadingTest(BaseTest):
                     'series' : 'A Series One',
                     'series_index': 1.0,
                     'tags':('Tag One', 'Tag Two'),
-                    'formats': (),
+                    'formats': ('FMT1',),
                     'rating': 4.0,
                     'identifiers': {'test':'one'},
                     'timestamp': datetime.datetime(2011, 9, 5, 21, 6,
@@ -86,6 +79,7 @@ class ReadingTest(BaseTest):
                     '#tags':('My Tag One', 'My Tag Two'),
                     '#yesno':True,
                     '#comments': '<div>My Comments One<p></p></div>',
+                    'size':9,
                 },
                 1  : {
                     'title': 'Title Two',
@@ -96,7 +90,7 @@ class ReadingTest(BaseTest):
                     'series_index': 2.0,
                     'rating': 6.0,
                     'tags': ('Tag One', 'News'),
-                    'formats':(),
+                    'formats':('FMT1', 'FMT2'),
                     'identifiers': {'test':'two'},
                     'timestamp': datetime.datetime(2011, 9, 6, 6, 0,
                         tzinfo=utc_tz),
@@ -115,12 +109,15 @@ class ReadingTest(BaseTest):
                     '#tags':('My Tag Two',),
                     '#yesno':False,
                     '#comments': '<div>My Comments Two<p></p></div>',
+                    'size':9,
 
                 },
         }
         for book_id, test in tests.iteritems():
             for field, expected_val in test.iteritems():
                 val = cache.field_for(field, book_id)
+                if isinstance(val, tuple) and 'authors' not in field and 'languages' not in field:
+                    val, expected_val = set(val), set(expected_val)
                 self.assertEqual(expected_val, val,
                         'Book id: %d Field: %s failed: %r != %r'%(
                             book_id, field, expected_val, val))
@@ -172,22 +169,46 @@ class ReadingTest(BaseTest):
         'Test get_metadata() returns the same data for both backends'
         from calibre.library.database2 import LibraryDatabase2
         old = LibraryDatabase2(self.library_path)
-        for i in xrange(1, 3):
-            old.add_format(i, 'txt%d'%i, StringIO(b'random%d'%i),
-                    index_is_id=True)
-            old.add_format(i, 'text%d'%i, StringIO(b'random%d'%i),
-                    index_is_id=True)
-
-        old_metadata = {i:old.get_metadata(i, index_is_id=True) for i in
+        old_metadata = {i:old.get_metadata(
+            i, index_is_id=True, get_cover=True, cover_as_data=True) for i in
                 xrange(1, 4)}
+        for mi in old_metadata.itervalues():
+            mi.format_metadata = dict(mi.format_metadata)
+            if mi.formats:
+                mi.formats = tuple(mi.formats)
+        old.conn.close()
         old = None
 
         cache = self.init_cache(self.library_path)
 
-        new_metadata = {i:cache.get_metadata(i) for i in xrange(1, 4)}
+        new_metadata = {i:cache.get_metadata(
+            i, get_cover=True, cover_as_data=True) for i in xrange(1, 4)}
         cache = None
         for mi2, mi1 in zip(new_metadata.values(), old_metadata.values()):
             self.compare_metadata(mi1, mi2)
+    # }}}
+
+    def test_get_cover(self): # {{{
+        'Test cover() returns the same data for both backends'
+        from calibre.library.database2 import LibraryDatabase2
+        old = LibraryDatabase2(self.library_path)
+        covers = {i: old.cover(i, index_is_id=True) for i in old.all_ids()}
+        old.conn.close()
+        old = None
+        cache = self.init_cache(self.library_path)
+        for book_id, cdata in covers.iteritems():
+            self.assertEqual(cdata, cache.cover(book_id), 'Reading of cover failed')
+            f = cache.cover(book_id, as_file=True)
+            self.assertEqual(cdata, f.read() if f else f, 'Reading of cover as file failed')
+            if cdata:
+                with open(cache.cover(book_id, as_path=True), 'rb') as f:
+                    self.assertEqual(cdata, f.read(), 'Reading of cover as path failed')
+            else:
+                self.assertEqual(cdata, cache.cover(book_id, as_path=True),
+                                 'Reading of null cover as path failed')
+        buf = BytesIO()
+        self.assertFalse(cache.copy_cover_to(99999, buf), 'copy_cover_to() did not return False for non-existent book_id')
+        self.assertFalse(cache.copy_cover_to(3, buf), 'copy_cover_to() did not return False for non-existent cover')
 
     # }}}
 
@@ -227,9 +248,14 @@ class ReadingTest(BaseTest):
             # User categories
             '@Good Authors:One', '@Good Series.good tags:two',
 
-            # TODO: Tests for searching the size and #formats columns and
-            # cover:true|false
+            # Cover/Formats
+            'cover:true', 'cover:false', 'formats:true', 'formats:false',
+            'formats:#>1', 'formats:#=1', 'formats:=fmt1', 'formats:=fmt2',
+            'formats:=fmt1 or formats:fmt2', '#formats:true', '#formats:false',
+            '#formats:fmt1', '#formats:fmt2', '#formats:fmt1 and #formats:fmt2',
+
         )}
+        old.conn.close()
         old = None
 
         cache = self.init_cache(self.library_path)
@@ -246,19 +272,74 @@ class ReadingTest(BaseTest):
         from calibre.library.database2 import LibraryDatabase2
         old = LibraryDatabase2(self.library_path)
         old_categories = old.get_categories()
+        old.conn.close()
         cache = self.init_cache(self.library_path)
-        import pprint
-        pprint.pprint(old_categories)
-        pprint.pprint(cache.get_categories())
+        new_categories = cache.get_categories()
+        self.assertEqual(set(old_categories), set(new_categories),
+            'The set of old categories is not the same as the set of new categories')
+
+        def compare_category(category, old, new):
+            for attr in ('name', 'original_name', 'id', 'count',
+                         'is_hierarchical', 'is_editable', 'is_searchable',
+                         'id_set', 'avg_rating', 'sort', 'use_sort_as_name',
+                         'tooltip', 'icon', 'category'):
+                oval, nval = getattr(old, attr), getattr(new, attr)
+                if (
+                    (category in {'rating', '#rating'} and attr in {'id_set', 'sort'}) or
+                    (category == 'series' and attr == 'sort') or # Sorting is wrong in old
+                    (category == 'identifiers' and attr == 'id_set') or
+                    (category == '@Good Series') or # Sorting is wrong in old
+                    (category == 'news' and attr in {'count', 'id_set'}) or
+                    (category == 'formats' and attr == 'id_set')
+                ):
+                    continue
+                self.assertEqual(oval, nval,
+                    'The attribute %s for %s in category %s does not match. Old is %r, New is %r'
+                                %(attr, old.name, category, oval, nval))
+
+        for category in old_categories:
+            old, new = old_categories[category], new_categories[category]
+            self.assertEqual(len(old), len(new),
+                'The number of items in the category %s is not the same'%category)
+            for o, n in zip(old, new):
+                compare_category(category, o, n)
 
     # }}}
 
-def tests():
-    return unittest.TestLoader().loadTestsFromTestCase(ReadingTest)
+    def test_get_formats(self): # {{{
+        'Test reading ebook formats using the format() method'
+        from calibre.library.database2 import LibraryDatabase2
+        from calibre.db.cache import NoSuchFormat
+        old = LibraryDatabase2(self.library_path)
+        ids = old.all_ids()
+        lf = {i:set(old.formats(i, index_is_id=True).split(',')) if old.formats(
+            i, index_is_id=True) else set() for i in ids}
+        formats = {i:{f:old.format(i, f, index_is_id=True) for f in fmts} for
+                   i, fmts in lf.iteritems()}
+        old.conn.close()
+        old = None
+        cache = self.init_cache(self.library_path)
+        for book_id, fmts in lf.iteritems():
+            self.assertEqual(fmts, set(cache.formats(book_id)),
+                             'Set of formats is not the same')
+            for fmt in fmts:
+                old = formats[book_id][fmt]
+                self.assertEqual(old, cache.format(book_id, fmt),
+                                 'Old and new format disagree')
+                f = cache.format(book_id, fmt, as_file=True)
+                self.assertEqual(old, f.read(),
+                                 'Failed to read format as file')
+                with open(cache.format(book_id, fmt, as_path=True,
+                                       preserve_filename=True), 'rb') as f:
+                    self.assertEqual(old, f.read(),
+                                 'Failed to read format as path')
+                with open(cache.format(book_id, fmt, as_path=True), 'rb') as f:
+                    self.assertEqual(old, f.read(),
+                                 'Failed to read format as path')
 
-def run():
-    unittest.TextTestRunner(verbosity=2).run(tests())
+        buf = BytesIO()
+        self.assertRaises(NoSuchFormat, cache.copy_format_to, 99999, 'X', buf, 'copy_format_to() failed to raise an exception for non-existent book')
+        self.assertRaises(NoSuchFormat, cache.copy_format_to, 1, 'X', buf, 'copy_format_to() failed to raise an exception for non-existent format')
 
-if __name__ == '__main__':
-    run()
+    # }}}
 
