@@ -6,7 +6,7 @@ import os, time, sys
 
 from calibre.constants import preferred_encoding, DEBUG
 from calibre import isbytestring, force_unicode
-from calibre.utils.icu import strcmp
+from calibre.utils.icu import sort_key
 
 from calibre.devices.usbms.books import Book as Book_
 from calibre.devices.usbms.books import CollectionsBookList
@@ -62,6 +62,7 @@ class Book(Book_):
         self.kobo_collections   = []
         self.kobo_series        = None
         self.kobo_series_number = None
+        self.can_put_on_shelves = True
 
         if thumbnail_name is not None:
             self.thumbnail = ImageWrapper(thumbnail_name)
@@ -141,7 +142,7 @@ class KTCollectionsBookList(CollectionsBookList):
                     if show_debug:
                         debug_print("KTCollectionsBookList:get_collections - adding book.device_collections", book.device_collections)
                 # If the book is not in the current library, we don't want to use the metadtaa for the collections
-                elif book.application_id is None:
+                elif book.application_id is None or not book.can_put_on_shelves:
 #                    debug_print("KTCollectionsBookList:get_collections - Book not in current library")
                     continue
                 else:
@@ -239,9 +240,8 @@ class KTCollectionsBookList(CollectionsBookList):
             if y is None:
                 return -1
             if isinstance(x, basestring) and isinstance(y, basestring):
-                c = strcmp(force_unicode(x), force_unicode(y))
-            else:
-                c = cmp(x, y)
+                x, y = sort_key(force_unicode(x)), sort_key(force_unicode(y))
+            c = cmp(x, y)
             if c != 0:
                 return c
             # same as above -- no sort_key needed here
