@@ -21,7 +21,7 @@ from calibre.ebooks.metadata.book.base import Metadata
 from calibre.utils.date import parse_only_date
 from calibre.utils.localization import canonicalize_lang
 
-class Worker(Thread): # Get details {{{
+class Worker(Thread):  # Get details {{{
 
     '''
     Get book details from amazons book page in a separate thread
@@ -43,12 +43,12 @@ class Worker(Thread): # Get details {{{
 
         months = {
                 'de': {
-            1 : ['jän'],
-            2 : ['februar'],
-            3 : ['märz'],
-            5 : ['mai'],
-            6 : ['juni'],
-            7 : ['juli'],
+            1: ['jän'],
+            2: ['februar'],
+            3: ['märz'],
+            5: ['mai'],
+            6: ['juni'],
+            7: ['juli'],
             10: ['okt'],
             12: ['dez']
             },
@@ -132,7 +132,7 @@ class Worker(Thread): # Get details {{{
                  text()="Détails sur le produit" or \
                  text()="Detalles del producto" or \
                  text()="Detalhes do produto" or \
-                 text()="登録情報"]/../div[@class="content"]
+                 starts-with(text(), "登録情報")]/../div[@class="content"]
             '''
         # Editor: is for Spanish
         self.publisher_xpath = '''
@@ -235,6 +235,12 @@ class Worker(Thread): # Get details {{{
             msg = 'Failed to parse amazon details page: %r'%self.url
             self.log.exception(msg)
             return
+        if self.domain == 'jp':
+            for a in root.xpath('//a[@href]'):
+                if 'black-curtain-redirect.html' in a.get('href'):
+                    self.url = 'http://amazon.co.jp'+a.get('href')
+                    self.log('Black curtain redirect found, following')
+                    return self.get_details()
 
         errmsg = root.xpath('//*[@id="errorMessage"]')
         if errmsg:
@@ -252,8 +258,8 @@ class Worker(Thread): # Get details {{{
             self.log.exception('Error parsing asin for url: %r'%self.url)
             asin = None
         if self.testing:
-            import tempfile
-            with tempfile.NamedTemporaryFile(prefix=asin + '_',
+            import tempfile, uuid
+            with tempfile.NamedTemporaryFile(prefix=(asin or str(uuid.uuid4()))+ '_',
                     suffix='.html', delete=False) as f:
                 f.write(raw)
             print ('Downloaded html for', asin, 'saved in', f.name)
@@ -270,7 +276,6 @@ class Worker(Thread): # Get details {{{
             self.log.exception('Error parsing authors for url: %r'%self.url)
             authors = []
 
-
         if not title or not authors or not asin:
             self.log.error('Could not find title/authors/asin for %r'%self.url)
             self.log.error('ASIN: %r Title: %r Authors: %r'%(asin, title,
@@ -302,7 +307,7 @@ class Worker(Thread): # Get details {{{
             self.log.exception('Error parsing series for url: %r'%self.url)
 
         try:
-            self.cover_url = self.parse_cover(root)
+            self.cover_url = self.parse_cover(root, raw)
         except:
             self.log.exception('Error parsing cover for url: %r'%self.url)
         mi.has_cover = bool(self.cover_url)
@@ -425,7 +430,6 @@ class Worker(Thread): # Get details {{{
         desc = re.sub(r'(?s)<!--.*?-->', '', desc)
         return sanitize_comments_html(desc)
 
-
     def parse_comments(self, root):
         ans = ''
         desc = root.xpath('//div[@id="ps-content"]/div[@class="content"]')
@@ -450,18 +454,24 @@ class Worker(Thread): # Get details {{{
                     ans = (s, i)
         return ans
 
-
-    def parse_cover(self, root):
-        imgs = root.xpath('//img[(@id="prodImage" or @id="original-main-image") and @src]')
+    def parse_cover(self, root, raw=b""):
+        imgs = root.xpath('//img[(@id="prodImage" or @id="original-main-image" or @id="main-image") and @src]')
+        if not imgs:
+            imgs = root.xpath('//div[@class="main-image-inner-wrapper"]/img[@src]')
         if imgs:
             src = imgs[0].get('src')
-            if '/no-image-avail' not in src:
+            if 'loading-' in src:
+                js_img = re.search(br'"largeImage":"(http://[^"]+)",',raw)
+                if js_img:
+                    src = js_img.group(1).decode('utf-8')
+            if ('/no-image-avail' not in src and 'loading-' not in src and '/no-img-sm' not in src):
+                self.log('Found image: %s' % src)
                 parts = src.split('/')
                 if len(parts) > 3:
                     bn = parts[-1]
                     sparts = bn.split('_')
                     if len(sparts) > 2:
-                        bn = sparts[0] + sparts[-1]
+                        bn = re.sub(r'\.\.jpg$', '.jpg', (sparts[0] + sparts[-1]))
                         return ('/'.join(parts[:-1]))+'/'+bn
 
     def parse_isbn(self, pd):
@@ -493,7 +503,7 @@ class Worker(Thread): # Get details {{{
     def parse_language(self, pd):
         for x in reversed(pd.xpath(self.language_xpath)):
             if x.tail:
-                raw = x.tail.strip()
+                raw = x.tail.strip().partition(',')[0].strip()
                 ans = self.lang_map.get(raw, None)
                 if ans:
                     return ans
@@ -516,13 +526,13 @@ class Amazon(Source):
 
     AMAZON_DOMAINS = {
             'com': _('US'),
-            'fr' : _('France'),
-            'de' : _('Germany'),
-            'uk' : _('UK'),
-            'it' : _('Italy'),
-            'jp' : _('Japan'),
-            'es' : _('Spain'),
-            'br' : _('Brazil'),
+            'fr': _('France'),
+            'de': _('Germany'),
+            'uk': _('UK'),
+            'it': _('Italy'),
+            'jp': _('Japan'),
+            'es': _('Spain'),
+            'br': _('Brazil'),
     }
 
     options = (
@@ -580,7 +590,7 @@ class Amazon(Source):
                     return domain, val
         return None, None
 
-    def get_book_url(self, identifiers): # {{{
+    def get_book_url(self, identifiers):  # {{{
         domain, asin = self.get_domain_and_asin(identifiers)
         if domain and asin:
             url = None
@@ -625,8 +635,7 @@ class Amazon(Source):
             mi.tags = list(map(fixcase, mi.tags))
         mi.isbn = check_isbn(mi.isbn)
 
-
-    def create_query(self, log, title=None, authors=None, identifiers={}, # {{{
+    def create_query(self, log, title=None, authors=None, identifiers={},  # {{{
             domain=None):
         if domain is None:
             domain = self.domain
@@ -636,8 +645,8 @@ class Amazon(Source):
             domain = idomain
 
         # See the amazon detailed search page to get all options
-        q = {   'search-alias' : 'aps',
-                'unfiltered' : '1',
+        q = {'search-alias': 'aps',
+             'unfiltered': '1',
             }
 
         if domain == 'com':
@@ -692,7 +701,7 @@ class Amazon(Source):
 
     # }}}
 
-    def get_cached_cover_url(self, identifiers): # {{{
+    def get_cached_cover_url(self, identifiers):  # {{{
         url = None
         domain, asin = self.get_domain_and_asin(identifiers)
         if asin is None:
@@ -705,14 +714,17 @@ class Amazon(Source):
         return url
     # }}}
 
-    def parse_results_page(self, root): # {{{
+    def parse_results_page(self, root):  # {{{
         from lxml.html import tostring
 
         matches = []
 
         def title_ok(title):
             title = title.lower()
-            for x in ('bulk pack', '[audiobook]', '[audio cd]'):
+            bad = ['bulk pack', '[audiobook]', '[audio cd]']
+            if self.domain == 'com':
+                bad.append('(spanish edition)')
+            for x in bad:
                 if x in title:
                     return False
             return True
@@ -739,13 +751,12 @@ class Amazon(Source):
                         matches.append(a.get('href'))
                     break
 
-
         # Keep only the top 5 matches as the matches are sorted by relevance by
         # Amazon so lower matches are not likely to be very relevant
         return matches[:5]
     # }}}
 
-    def identify(self, log, result_queue, abort, title=None, authors=None, # {{{
+    def identify(self, log, result_queue, abort, title=None, authors=None,  # {{{
             identifiers={}, timeout=30):
         '''
         Note this method will retry without identifiers automatically if no
@@ -783,7 +794,6 @@ class Amazon(Source):
                 log.exception(msg)
             return as_unicode(msg)
 
-
         raw = clean_ascii_chars(xml_to_unicode(raw,
             strip_encoding_pats=True, resolve_entities=True)[0])
 
@@ -813,7 +823,6 @@ class Amazon(Source):
                     # The error is almost always a not found error
                     found = False
 
-
         if found:
             matches = self.parse_results_page(root)
 
@@ -851,8 +860,8 @@ class Amazon(Source):
         return None
     # }}}
 
-    def download_cover(self, log, result_queue, abort, # {{{
-            title=None, authors=None, identifiers={}, timeout=30):
+    def download_cover(self, log, result_queue, abort,  # {{{
+            title=None, authors=None, identifiers={}, timeout=30, get_best_cover=False):
         cached_url = self.get_cached_cover_url(identifiers)
         if cached_url is None:
             log.info('No cached cover found, running identify')
@@ -888,32 +897,44 @@ class Amazon(Source):
             log.exception('Failed to download cover from:', cached_url)
     # }}}
 
-if __name__ == '__main__': # tests {{{
+if __name__ == '__main__':  # tests {{{
     # To run these test use: calibre-debug -e
     # src/calibre/ebooks/metadata/sources/amazon.py
     from calibre.ebooks.metadata.sources.test import (test_identify_plugin,
             isbn_test, title_test, authors_test, comments_test, series_test)
-    com_tests = [ # {{{
+    com_tests = [  # {{{
 
-            ( # Series
+            (  # Has a spanish edition
+             {'title':'11/22/63'},
+             [title_test('11/22/63: A Novel', exact=True), authors_test(['Stephen King']),]
+             ),
+
+            (  # + in title and uses id="main-image" for cover
+             {'title':'C++ Concurrency in Action'},
+             [title_test('C++ Concurrency in Action: Practical Multithreading',
+                         exact=True),
+              ]
+             ),
+
+            (  # Series
                 {'identifiers':{'amazon':'0756407117'}},
                 [title_test(
-                "Throne of the Crescent Moon"
-                , exact=True), series_test('Crescent Moon Kingdoms', 1),
+                "Throne of the Crescent Moon",
+                exact=True), series_test('Crescent Moon Kingdoms', 1),
                 comments_test('Makhslood'),
                 ]
             ),
 
-            ( # Different comments markup, using Book Description section
+            (  # Different comments markup, using Book Description section
                 {'identifiers':{'amazon':'0982514506'}},
                 [title_test(
-                "Griffin's Destiny: Book Three: The Griffin's Daughter Trilogy"
-                , exact=True),
+                "Griffin's Destiny: Book Three: The Griffin's Daughter Trilogy",
+                exact=True),
                 comments_test('Jelena'), comments_test('Leslie'),
                 ]
             ),
 
-            ( # # in title
+            (  # # in title
                 {'title':'Expert C# 2008 Business Objects',
                     'authors':['Lhotka']},
                 [title_test('Expert C# 2008 Business Objects', exact=True),
@@ -929,13 +950,13 @@ if __name__ == '__main__': # tests {{{
 
             ),
 
-            ( # Sophisticated comment formatting
+            (  # Sophisticated comment formatting
                 {'identifiers':{'isbn': '9781416580829'}},
                 [title_test('Angels & Demons - Movie Tie-In: A Novel',
                     exact=True), authors_test(['Dan Brown'])]
             ),
 
-            ( # No specific problems
+            (  # No specific problems
                 {'identifiers':{'isbn': '0743273567'}},
                 [title_test('The great gatsby', exact=True),
                     authors_test(['F. Scott Fitzgerald'])]
@@ -948,9 +969,9 @@ if __name__ == '__main__': # tests {{{
 
             ),
 
-    ] # }}}
+    ]  # }}}
 
-    de_tests = [ # {{{
+    de_tests = [  # {{{
             (
                 {'identifiers':{'isbn': '3548283519'}},
                 [title_test('Wer Wind Sät: Der Fünfte Fall Für Bodenstein Und Kirchhoff',
@@ -958,9 +979,9 @@ if __name__ == '__main__': # tests {{{
                  ]
 
             ),
-    ] # }}}
+    ]  # }}}
 
-    it_tests = [ # {{{
+    it_tests = [  # {{{
             (
                 {'identifiers':{'isbn': '8838922195'}},
                 [title_test('La briscola in cinque',
@@ -968,9 +989,9 @@ if __name__ == '__main__': # tests {{{
                  ]
 
             ),
-    ] # }}}
+    ]  # }}}
 
-    fr_tests = [ # {{{
+    fr_tests = [  # {{{
             (
                 {'identifiers':{'isbn': '2221116798'}},
                 [title_test('L\'étrange voyage de Monsieur Daldry',
@@ -978,9 +999,9 @@ if __name__ == '__main__': # tests {{{
                  ]
 
             ),
-    ] # }}}
+    ]  # }}}
 
-    es_tests = [ # {{{
+    es_tests = [  # {{{
             (
                 {'identifiers':{'isbn': '8483460831'}},
                 [title_test('Tiempos Interesantes',
@@ -988,23 +1009,28 @@ if __name__ == '__main__': # tests {{{
                  ]
 
             ),
-    ] # }}}
+    ]  # }}}
 
-    jp_tests = [ # {{{
-            ( # isbn -> title, authors
-                {'identifiers':{'isbn': '9784101302720' }},
+    jp_tests = [  # {{{
+            (  # Adult filtering test
+             {'identifiers':{'isbn':'4799500066'}},
+             [title_test(u'Ｂｉｔｃｈ Ｔｒａｐ'),]
+            ),
+
+            (  # isbn -> title, authors
+                {'identifiers':{'isbn': '9784101302720'}},
                 [title_test(u'精霊の守り人',
                     exact=True), authors_test([u'上橋 菜穂子'])
                  ]
             ),
-            ( # title, authors -> isbn (will use Shift_JIS encoding in query.)
+            (  # title, authors -> isbn (will use Shift_JIS encoding in query.)
                 {'title': u'考えない練習',
                  'authors': [u'小池 龍之介']},
                 [isbn_test('9784093881067'), ]
             ),
-    ] # }}}
+    ]  # }}}
 
-    br_tests = [ # {{{
+    br_tests = [  # {{{
             (
                 {'title':'Guerra dos Tronos'},
                 [title_test('A Guerra dos Tronos - As Crônicas de Gelo e Fogo',
@@ -1012,7 +1038,7 @@ if __name__ == '__main__': # tests {{{
                  ]
 
             ),
-    ] # }}}
+    ]  # }}}
 
     def do_test(domain, start=0, stop=None):
         tests = globals().get(domain+'_tests')
