@@ -177,11 +177,13 @@ class SendToDeviceAction(InterfaceAction):
 class ConnectShareAction(InterfaceAction):
 
     name = 'Connect Share'
-    action_spec = (_('Connect/share'), 'connect_share.png', None, None)
+    action_spec = (_('Connect/share'), 'connect_share.png',
+                   _('Share books using a web server or email. Connect to special devices, etc.'), None)
     popup_type = QToolButton.InstantPopup
 
     def genesis(self):
         self.share_conn_menu = ShareConnMenu(self.gui)
+        self.share_conn_menu.aboutToShow.connect(self.set_smartdevice_action_state)
         self.share_conn_menu.toggle_server.connect(self.toggle_content_server)
         self.share_conn_menu.control_smartdevice.connect(self.control_smartdevice)
         self.share_conn_menu.config_email.connect(partial(
@@ -258,7 +260,10 @@ class ConnectShareAction(InterfaceAction):
             show_port = True
         else:
             all_ips = get_all_ip_addresses()
-            if len(all_ips) > 3:
+            if len(all_ips) == 0:
+                formatted_addresses = _('Still looking for IP addresses')
+                show_port = False
+            elif len(all_ips) > 3:
                 formatted_addresses = _('Many IP addresses. See Start/Stop dialog.')
                 show_port = False
             else:
