@@ -23,10 +23,38 @@ from calibre.db.errors import NoSuchFormat
 class EditMetadataAction(InterfaceAction):
 
     name = 'Edit Metadata'
-    action_spec = (_('Edit metadata'), 'edit_input.png', None, _('E'))
+    action_spec = (_('Edit metadata'), 'edit_input.png', _('Change the title/author/cover etc. of books'), _('E'))
     action_type = 'current'
     action_add_menu = True
 
+    accepts_drops = True
+
+    def accept_enter_event(self, event, mime_data):
+        if mime_data.hasFormat("application/calibre+from_library"):
+            return True
+        return False
+
+    def accept_drag_move_event(self, event, mime_data):
+        if mime_data.hasFormat("application/calibre+from_library"):
+            return True
+        return False
+
+    def drop_event(self, event, mime_data):
+        mime = 'application/calibre+from_library'
+        if mime_data.hasFormat(mime):
+            self.dropped_ids = tuple(map(int, str(mime_data.data(mime)).split()))
+            QTimer.singleShot(1, self.do_drop)
+            return True
+        return False
+
+    def do_drop(self):
+        book_ids = self.dropped_ids
+        del self.dropped_ids
+        if book_ids:
+            db = self.gui.library_view.model().db
+            rows = [db.row(i) for i in book_ids]
+            self.edit_metadata_for(rows, book_ids)
+
     def genesis(self):
         md = self.qaction.menu()
         cm = partial(self.create_menu_action, md)
@@ -186,18 +214,23 @@ class EditMetadataAction(InterfaceAction):
         Edit metadata of selected books in library.
         '''
         rows = self.gui.library_view.selectionModel().selectedRows()
-        previous = self.gui.library_view.currentIndex()
         if not rows or len(rows) == 0:
             d = error_dialog(self.gui, _('Cannot edit metadata'),
                              _('No books selected'))
             d.exec_()
             return
-
-        if bulk or (bulk is None and len(rows) > 1):
-            return self.edit_bulk_metadata(checked)
-
         row_list = [r.row() for r in rows]
+        m = self.gui.library_view.model()
+        ids = [m.id(r) for r in rows]
+        self.edit_metadata_for(row_list, ids, bulk=bulk)
+
+    def edit_metadata_for(self, rows, book_ids, bulk=None):
+        previous = self.gui.library_view.currentIndex()
+        if bulk or (bulk is None and len(rows) > 1):
+            return self.do_edit_bulk_metadata(rows, book_ids)
+
         current_row = 0
+        row_list = rows
 
         if len(row_list) == 1:
             cr = row_list[0]
@@ -242,12 +275,11 @@ class EditMetadataAction(InterfaceAction):
             db = self.gui.library_view.model().db
             view.view_format(db.row(id_), fmt)
 
-
     def edit_bulk_metadata(self, checked):
         '''
         Edit metadata of selected books in library in bulk.
         '''
-        rows = [r.row() for r in \
+        rows = [r.row() for r in
                 self.gui.library_view.selectionModel().selectedRows()]
         m = self.gui.library_view.model()
         ids = [m.id(r) for r in rows]
@@ -256,6 +288,9 @@ class EditMetadataAction(InterfaceAction):
                     _('No books selected'))
             d.exec_()
             return
+        self.do_edit_bulk_metadata(rows, ids)
+
+    def do_edit_bulk_metadata(self, rows, book_ids):
         # Prevent the TagView from updating due to signals from the database
         self.gui.tags_view.blockSignals(True)
         changed = False
@@ -278,7 +313,7 @@ class EditMetadataAction(InterfaceAction):
             self.gui.tags_view.recount()
             if self.gui.cover_flow:
                 self.gui.cover_flow.dataChanged()
-            self.gui.library_view.select_rows(ids)
+            self.gui.library_view.select_rows(book_ids)
 
     # Merge books {{{
     def merge_books(self, safe_merge=False, merge_only_formats=False):
@@ -434,45 +469,39 @@ class EditMetadataAction(InterfaceAction):
         if not had_orig_cover and dest_cover:
             db.set_cover(dest_id, dest_cover)
 
-        for key in db.field_metadata: #loop thru all defined fields
-          if db.field_metadata[key]['is_custom']:
-            colnum = db.field_metadata[key]['colnum']
+        for key in db.field_metadata:  # loop thru all defined fields
+            fm = db.field_metadata[key]
+            if not fm['is_custom']:
+                continue
+            dt = fm['datatype']
+            colnum = fm['colnum']
             # Get orig_dest_comments before it gets changed
-            if db.field_metadata[key]['datatype'] == 'comments':
-              orig_dest_value = db.get_custom(dest_id, num=colnum, index_is_id=True)
+            if dt == 'comments':
+                orig_dest_value = db.get_custom(dest_id, num=colnum, index_is_id=True)
+
             for src_id in src_ids:
-              dest_value = db.get_custom(dest_id, num=colnum, index_is_id=True)
-              src_value = db.get_custom(src_id, num=colnum, index_is_id=True)
-              if db.field_metadata[key]['datatype'] == 'comments':
-                if src_value and src_value != orig_dest_value:
-                  if not dest_value:
+                dest_value = db.get_custom(dest_id, num=colnum, index_is_id=True)
+                src_value = db.get_custom(src_id, num=colnum, index_is_id=True)
+                if (dt == 'comments' and src_value and src_value != orig_dest_value):
+                    if not dest_value:
+                        db.set_custom(dest_id, src_value, num=colnum)
+                    else:
+                        dest_value = unicode(dest_value) + u'\n\n' + unicode(src_value)
+                        db.set_custom(dest_id, dest_value, num=colnum)
+                if (dt in {'bool', 'int', 'float', 'rating', 'datetime'} and dest_value is None):
                     db.set_custom(dest_id, src_value, num=colnum)
-                  else:
-                    dest_value = unicode(dest_value) + u'\n\n' + unicode(src_value)
+                if (dt == 'series' and not dest_value and src_value):
+                    src_index = db.get_custom_extra(src_id, num=colnum, index_is_id=True)
+                    db.set_custom(dest_id, src_value, num=colnum, extra=src_index)
+                if (dt == 'enumeration' or (dt == 'text' and not fm['is_multiple']) and not dest_value):
+                    db.set_custom(dest_id, src_value, num=colnum)
+                if (dt == 'text' and fm['is_multiple'] and src_value):
+                    if not dest_value:
+                        dest_value = src_value
+                    else:
+                        dest_value.extend(src_value)
                     db.set_custom(dest_id, dest_value, num=colnum)
-              if db.field_metadata[key]['datatype'] in \
-                ('bool', 'int', 'float', 'rating', 'datetime') \
-                and dest_value is None:
-                db.set_custom(dest_id, src_value, num=colnum)
-              if db.field_metadata[key]['datatype'] == 'series' \
-                and not dest_value:
-                if src_value:
-                  src_index = db.get_custom_extra(src_id, num=colnum, index_is_id=True)
-                  db.set_custom(dest_id, src_value, num=colnum, extra=src_index)
-              if (db.field_metadata[key]['datatype'] == 'enumeration' or
-                        (db.field_metadata[key]['datatype'] == 'text' and
-                         not db.field_metadata[key]['is_multiple'])
-                    and not dest_value):
-                db.set_custom(dest_id, src_value, num=colnum)
-              if db.field_metadata[key]['datatype'] == 'text' \
-                and db.field_metadata[key]['is_multiple']:
-                if src_value:
-                  if not dest_value:
-                    dest_value = src_value
-                  else:
-                    dest_value.extend(src_value)
-                  db.set_custom(dest_id, dest_value, num=colnum)
-        # }}}
+    # }}}
 
     def edit_device_collections(self, view, oncard=None):
         model = view.model()
@@ -480,8 +509,8 @@ class EditMetadataAction(InterfaceAction):
         d = DeviceCategoryEditor(self.gui, tag_to_match=None, data=result, key=sort_key)
         d.exec_()
         if d.result() == d.Accepted:
-            to_rename = d.to_rename # dict of new text to old ids
-            to_delete = d.to_delete # list of ids
+            to_rename = d.to_rename  # dict of new text to old ids
+            to_delete = d.to_delete  # list of ids
             for old_id, new_name in to_rename.iteritems():
                 model.rename_collection(old_id, new_name=unicode(new_name))
             for item in to_delete:
@@ -550,7 +579,6 @@ class EditMetadataAction(InterfaceAction):
             self.apply_pd.value += 1
         QTimer.singleShot(50, self.do_one_apply)
 
-
     def apply_mi(self, book_id, mi):
         db = self.gui.current_db
 
