@@ -13,8 +13,8 @@ from calibre.gui2 import error_dialog
 class ShowBookDetailsAction(InterfaceAction):
 
     name = 'Show Book Details'
-    action_spec = (_('Show book details'), 'dialog_information.png', None,
-            _('I'))
+    action_spec = (_('Show book details'), 'dialog_information.png',
+                   _('Show the detailed metadata for the current book in a separate window'), _('I'))
     dont_add_to = frozenset(['context-menu-device'])
     action_type = 'current'
 
