@@ -197,8 +197,7 @@ class TemplateHighlighter(QSyntaxHighlighter):
 
 class TemplateDialog(QDialog, Ui_TemplateDialog):
 
-    def __init__(self, parent, text, mi=None, fm=None, color_field=None,
-                 rule_kind='color'):
+    def __init__(self, parent, text, mi=None, fm=None, color_field=None):
         QDialog.__init__(self, parent)
         Ui_TemplateDialog.__init__(self)
         self.setupUi(self)
@@ -265,8 +264,7 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         self.function.setCurrentIndex(0)
         self.function.currentIndexChanged[str].connect(self.function_changed)
         self.textbox_changed()
-        self.rule = (rule_kind, None, '')
-        self.rule_kind = rule_kind
+        self.rule = (None, '')
 
         tt = _('Template language tutorial')
         self.template_tutorial.setText(
@@ -325,5 +323,5 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                     _('The template box cannot be empty'), show=True)
                 return
 
-        self.rule = (self.rule_kind, unicode(self.colored_field.currentText()), txt)
+        self.rule = (unicode(self.colored_field.currentText()), txt)
         QDialog.accept(self)
