@@ -94,6 +94,9 @@ class LibraryViewMixin(object): # {{{
                 v = self.current_view()
                 if hasattr(v, 'set_current_row'):
                     v.set_current_row(0)
+                    if v is self.library_view and v.row_count() == 0:
+                        self.book_details.reset_info()
+
 
     # }}}
 
@@ -269,6 +272,8 @@ class LayoutMixin(object): # {{{
                 self.iactions['Remove Books'].remove_format_by_id)
         self.book_details.save_specific_format.connect(
                 self.iactions['Save To Disk'].save_library_format_by_ids)
+        self.book_details.restore_specific_format.connect(
+            self.iactions['Remove Books'].restore_format)
         self.book_details.view_device_book.connect(
                 self.iactions['View'].view_device_book)
 
