@@ -255,17 +255,11 @@ class PocketBookPro912(PocketBook):
 
 class iPhone(Device):
 
-    name = 'iPhone/iTouch'
+    name = 'iPhone/iPad/iPod Touch'
     output_format = 'EPUB'
     manufacturer = 'Apple'
     id = 'iphone'
     supports_color = True
-    output_profile = 'ipad'
-
-class iPad(iPhone):
-
-    name = 'iPad'
-    id = 'ipad'
     output_profile = 'ipad3'
 
 class Android(Device):
